local about_file = "about.txt"
local license_dir = "licenses/"
local license_manifest = "manifest.csv"
local about_scroll_animation = nil

local module_revs = {["GUI"] = cREVISION_STRING,
					 ["User Interface"] = "N/A",
					 ["Power Module"] = "N/A",
					 ["Softener Module"] = "N/A",
           ["Second Bootloader"] = "N/A"}
local licenses_table = {}

local function load_licenses_manifest()
	local pFile = assert(io.open(license_dir..license_manifest))
	licenses_table = {}
	
	if pFile then
		for line in pFile:lines() do
			local entry = ParseCSVLine(line, ',')
			if entry[1] ~= "PACKAGE" then
				local project = {}
				project["package"]= entry[1]
				project["version"] = entry[2]
				project["license"] = entry[3]
				project["file"] = entry[4]
				licenses_table[#licenses_table+1] = project
			end
		end
		pFile:close()
	end
end

function do_rev_info(revisions)

	module_revs["User Interface"] = string.format("%c%c%c%c%c%c%c%c%c",revisions[1],revisions[2],revisions[3],revisions[4],revisions[5],
                                                           revisions[6],revisions[7],revisions[8],revisions[9])
	module_revs["Power Module"] = string.format("%c%c%c%c%c%c%c%c%c",revisions[10],revisions[11],revisions[12],revisions[13],revisions[14],
                                                          revisions[15],revisions[16],revisions[17],revisions[18])
	module_revs["Softener Module"] = string.format("%c%c%c%c%c%c%c%c%c",revisions[19],revisions[20],revisions[21],revisions[22],revisions[23],
                                                             revisions[24],revisions[25],revisions[26],revisions[27])

end

function update_about_screen()
	local about_text = ""
	local data = {}
	

	data = gre.get_data("service_page_7.bootloader_present_text", "service_page_7.about_text.grd_width", 
	                    "control_font", "service_page_7.license_table.grd_yoffset", 
	                    "service_page_7.license_table.grd_yoffset")
	
	about_text = about_text .."IM S/W Ver: "..module_revs["User Interface"].."\n"
	about_text = about_text .."PM S/W Ver: "..module_revs["Power Module"].."\n"
	about_text = about_text .."WS S/W Ver: "..module_revs["Softener Module"].."\n"
--  about_text = about_text .."2nd Bootloader: "..data["service_page_7.bootloader_present_text"]
	
  data["service_page_7.about_text.text"] = about_text
	
	--Calculate Text Window Height	
	data["service_page_7.about_text.text_height"] = calc_Height(data["service_page_7.about_text.text"], data["control_font"], 20, data["service_page_7.about_text.grd_width"])
	
	load_licenses_manifest()
	
	local project_count = 0
	for idx,project in ipairs(licenses_table) do
		project_count = project_count + 1
		local table_text = string.format("%s %s\n%s",project["package"],project["version"],project["license"])
		data[string.format("service_page_7.license_table.licenses.%d.1",idx)] = table_text
		data[string.format("service_page_7.license_table.button_image.%d.2",idx)] = "images/right_arrow_up.png"
	end
	
	--Edit Table size and position
	--data["service_page_7.license_table.grd_height"] = 80*project_count
	data["service_page_7.license_table.grd_y"] = data["service_page_7.about_text.text_height"] 
	data["service_page_7.license_table.grd_height"] = 270 - data["service_page_7.license_table.grd_y"]
	gre.set_table_attrs("service_page_7.license_table", {rows = project_count})
	
	--Reset View
	data["service_page_7.about_text.grd_y"] = 0
	data["service_page_7.about_text.scroll_target"] = 0
	
	if data["service_page_7.license_table.grd_yoffset"] >= 0 then
	  data["service_page_7.about_scroll_up.grd_hidden"] = 1
	end
	
	if data["service_page_7.license_table.grd_yoffset"] <= -2320 then
    data["service_page_7.about_scroll_down.grd_hidden"] = 1
  end
	
	gre.set_data(data)
end

function about_text_scroll(mapargs)
	local data = {}
	data = gre.get_data("service_page_7.about_text.scroll_target", "service_page_7.about_text.text_height",
						 "service_page_7.grd_height", "service_page_7.license_table.grd_height", 
						 "service_page_7.license_table.grd_yoffset", "service_page_7.license_table.grd_yoffset")
	

	if mapargs.dir == "up" then
	  data["service_page_7.license_table.grd_yoffset"] = data["service_page_7.license_table.grd_yoffset"] + 80
    if data["service_page_7.license_table.grd_yoffset"] < 0 then
      data["service_page_7.about_scroll_down.grd_hidden"] = 0
  	else
		 data["service_page_7.about_scroll_up.grd_hidden"] = 1
		end
	elseif mapargs.dir == "down" then
	  data["service_page_7.license_table.grd_yoffset"] = data["service_page_7.license_table.grd_yoffset"] - 80
	  if data["service_page_7.license_table.grd_yoffset"] > -2320 then
	    data["service_page_7.about_scroll_up.grd_hidden"] = 0
    else
      data["service_page_7.about_scroll_down.grd_hidden"] = 1
    end
	end
	
	gre.set_data(data)

end



function license_text_scroll(mapargs) 
	local data = {}
	data = gre.get_data("license_layer.license_text.scroll_target", "license_layer.license_text.text_height", "license_layer.license_text.grd_height", "license_layer.license_text.scroll_position")
	
	local max_scroll = 0
	if data["license_layer.license_text.text_height"] > data["license_layer.license_text.grd_height"] then
		max_scroll = data["license_layer.license_text.grd_height"] - data["license_layer.license_text.text_height"]
	end
	
	if mapargs.dir == "up" then
		data["license_layer.license_text.scroll_position"] = data["license_layer.license_text.scroll_position"] + 90
		if data["license_layer.license_text.scroll_position"] > 0 then
			data["license_layer.license_text.scroll_position"] = 0
		end
	elseif mapargs.dir == "down" then
		data["license_layer.license_text.scroll_position"] = data["license_layer.license_text.scroll_position"] - 90
		if data["license_layer.license_text.scroll_position"] < max_scroll then
			data["license_layer.license_text.scroll_position"] = max_scroll
		end
	end
	gre.set_data(data)
	gre.send_event("license_scroll")
end

function show_license(mapargs)
	local data = {}
	local license_text = ""
	data = gre.get_data("control_font","license_layer.license_text.grd_width")
	
	for w in licenses_table[mapargs.context_row]["file"]:gmatch("%S+") do
		local license_path = string.format(license_dir.."%s//%s",licenses_table[mapargs.context_row]["package"],w)
		local pFile = io.open(license_path, "r")
		if pFile then
			license_text = license_text .. pFile:read("*all") .. "\n"
			pFile:close()
		end
	end
	
	data["license_layer.license_text.text"] = license_text
	data["license_layer.license_text.scroll_position"] = 0
	
	--Calculate Text Window Height	
	data["license_layer.license_text.text_height"] = 15500 --calc_Height(license_text, data["control_font"], 12, data["license_layer.license_text.grd_width"])
			
	data["Service_screen.license_layer.grd_hidden"] = 0
	data["Service_screen.service_page_7.grd_hidden"] = 1
	data["Service_screen.bottom_buttons_service.grd_hidden"] = 1
	gre.set_data(data)
end

function exit_license(mapargs)
	local data = {}
	data["Service_screen.license_layer.grd_hidden"] = 1
	data["Service_screen.service_page_7.grd_hidden"] = 0
	data["Service_screen.bottom_buttons_service.grd_hidden"] = 0
	gre.set_data(data)
end
