-- Global Definitions
bg_color = 0x231f20
select_color = 0x6cbf66
highlight_yellow = 0xffc627

cGUI_STATE_NOSTATE        = 0
cGUI_STATE_INITIALIZING   = 1
cGUI_STATE_WAITING        = 2
cGUI_STATE_IDLE           = 3
cGUI_STATE_GEN_SOLUTION   = 4
cGUI_STATE_SOFTENER_REGEN = 5
cGUI_STATE_ERROR_MODE     = 6 
cGUI_STATE_WARNING_MODE   = 7

cAPP_STATE_STARTUP            = 0
cAPP_STATE_IDLE               = 1
cAPP_STATE_WAIT               = 2
cAPP_STATE_REGEN              = 3
cAPP_STATE_SOLUTION           = 4
cAPP_STATE_INFO               = 5
cAPP_STATE_QUANTITY_DISPENSED = 6
cAPP_STATE_TIMEOUT            = 7
cAPP_STATE_ERROR              = 8
cAPP_STATE_WARNING            = 9
cAPP_STATE_NONE               = 0xFF

cPASCAL_LOCKOUT_ENABLE    = 0xFFFE
cPASCAL_LOCKOUT_NO_PROPEL = 0xFFFD
cPASCAL_LOCKOUT_DISABLE   = 0x000D
cPASCAL_ABOUT_REQ         = 0x0000
cPASCAL_REFRESH_REQ       = 0x0001
cPASCAL_SERVICE_SCREEN    = 0x0002
cPASCAL_WEAR_RESET_RESP   = 0x0003

cNO_ERRORS                       = 0x00000000 
cINVALID_SWITCHES_ERROR          = 0x00000001 --SYS100
cPOWER_SUPPLY_VOLTAGE_ERROR      = 0x00000002 --PS100
cWATER_TEMP_TOO_HIGH_ERROR       = 0x00000004 --EC100
cWATER_TEMP_TOO_LOW_ERROR        = 0x00000008 --EC110
--cATX_12V_NOK_ERROR               = 0x00000010 --PS110
cGENERATION_TIMEOUT_ERROR        = 0x00000020 --SYS200
--cCONDUCTIVITY_CAL_ERROR          = 0x00000040 --PRB130
cPH_TOO_LOW                      = 0x00000080 --PRB100
cPH_TOO_HIGH                     = 0x00000100 --PRB110
cREPLACE_PH_PROBE                = 0x00000200 --PRB120
cPCB_AMBIENT_TOO_HIGH_ERROR      = 0x00000400 --PM100
cNO_WATER_FLOW_ERROR             = 0x00000800 --SYS120
cPM_COMMUNICATIONS_ERROR         = 0x00001000 --COM100
cWS_COMMUNICATIONS_ERROR         = 0x00002000 --COM110
cWS_SWITCH_ERROR                 = 0x00004000 --WS100
cWS_MOTOR_ERROR                  = 0x00008000 --WS200
cWS_REGEN_ERROR                  = 0x00010000 --WS500
cWS_BRINE_FILL_ERROR             = 0x00020000 --WS400
cWS_BRINE_SENSOR_ERROR           = 0x00040000 --WS300
cTSK_STARTUP_ERROR               = 0x00080000 --TSK100
cLEVEL_SWITCH_UNPLUGGED          = 0x00100000 --SYS130
--cINVALID_HARDNESS_CALCULATION    = 0x00200000 --SYS140
cE_TAPE_VOLTAGE_TOO_HIGH         = 0x00400000 --SYS150
cPH_SPI_BUS_READ_ERROR           = 0x00800000 --SYS160
cE_TAPE_VOLTAGE_TOO_LOW          = 0x01000000 --SYS170
cTOTAL_CELL_CURRENT_TOO_LOW      = 0x02000000 --CEL100
cANOLYTE_CELL_CURRENT_TOO_LOW    = 0x04000000 --CEL110
cCATHOLYTE_CELL_CURRENT_TOO_LOW  = 0x08000000 --CEL120
cANOLYTE_CELL_CURRENT_TOO_HIGH   = 0x10000000 --CEL130
cCATHOLYTE_CELL_CURRENT_TOO_HIGH = 0x20000000 --CEL140
cBAD_CONDUCTIVITY_PROBE          = 0x40000000 --PRB200
cANOLYTE_CELL_SHORT              = 0x80000000 --CEL150
cCATHOLYTE_CELL_SHORT            = 0x00000001 --CEL160
cTOTAL_CELL_CURRENT_TOO_HIGH     = 0x00000002 --CEL170
cANOLYTE_CELL_FET_SHORT          = 0x00000004 --PM200
cCATHOLYTE_CELL_FET_SHORT        = 0x00000008 --PM210
cCLEAR_ALL_ERRORS                = 0xFFFFFFFFFFFFFFFF

cHOT_WATER_DETECTED = 0x00000001
cLOW_SALT_DETECTED  = 0x00000002

cPASCAL_SIGN_POSITIVE = 0
cPASCAL_SIGN_NEGATIVE = 1

cWEAR_COMPONENT_ECELL    = 0
cWEAR_COMPONENT_MANIFOLD = 1
cWEAR_COMPONENT_PH       = 2

cWEAR_CONFIRM_INACTIVE  = 0
cWEAR_CONFIRM_WAITING   = 1
cWEAR_CONFIRM_REQUESTED = 2
cWEAR_CONFIRM_ACCEPTED  = 3
cWEAR_CONFIRM_DENIED    = 4

cNO_WEAR_CODE_ACTIVE = 0x00000000
cECELL_WEAR_CODE     = 0x00000001
cMAN_WEAR_CODE       = 0x00000002
cPH_WEAR_CODE        = 0x00000004

local fault_table = { { fault = cINVALID_SWITCHES_ERROR,          fault_text = "SYS100" },
                      { fault = cPOWER_SUPPLY_VOLTAGE_ERROR,      fault_text = "PS100"  },
                      { fault = cWATER_TEMP_TOO_HIGH_ERROR,       fault_text = "EC100"  },
                      { fault = cWATER_TEMP_TOO_LOW_ERROR,        fault_text = "EC110"  },
                   -- { fault = cATX_12V_NOK_ERROR,               fault_text = "PS110"  },
                   -- { fault = cCONDUCTIVITY_CAL_ERROR,          fault_text = "PRB130" },
                      { fault = cPH_TOO_LOW,                      fault_text = "PRB100" },
                      { fault = cPH_TOO_HIGH,                     fault_text = "PRB110" },
                      { fault = cREPLACE_PH_PROBE,                fault_text = "PRB120" },
                      { fault = cNO_WATER_FLOW_ERROR,             fault_text = "SYS120" },
                      { fault = cPM_COMMUNICATIONS_ERROR,         fault_text = "COM100" },
                      { fault = cWS_COMMUNICATIONS_ERROR,         fault_text = "COM110" },
                      { fault = cWS_SWITCH_ERROR,                 fault_text = "WS100"  },
                      { fault = cWS_MOTOR_ERROR,                  fault_text = "WS200"  },
                      { fault = cWS_REGEN_ERROR,                  fault_text = "WS500"  },
                      { fault = cWS_BRINE_FILL_ERROR,             fault_text = "WS400"  },
                      { fault = cWS_BRINE_SENSOR_ERROR,           fault_text = "WS300"  },
                      { fault = cTSK_STARTUP_ERROR,               fault_text = "TSK100" },
                      { fault = cLEVEL_SWITCH_UNPLUGGED,          fault_text = "SYS130" },
                   -- { fault = cINVALID_HARDNESS_CALCULATION,    fault_text = "SYS140" },
                      { fault = cE_TAPE_VOLTAGE_TOO_HIGH,         fault_text = "SYS150" },
                      { fault = cPH_SPI_BUS_READ_ERROR,           fault_text = "SYS160" },
                      { fault = cE_TAPE_VOLTAGE_TOO_LOW,          fault_text = "SYS170" },
                      { fault = cTOTAL_CELL_CURRENT_TOO_LOW,      fault_text = "CEL100" },
                      { fault = cANOLYTE_CELL_CURRENT_TOO_LOW,    fault_text = "CEL110" },
                      { fault = cCATHOLYTE_CELL_CURRENT_TOO_LOW,  fault_text = "CEL120" },
                      { fault = cANOLYTE_CELL_CURRENT_TOO_HIGH,   fault_text = "CEL130" },
                      { fault = cCATHOLYTE_CELL_CURRENT_TOO_HIGH, fault_text = "CEL140" },
                      { fault = cBAD_CONDUCTIVITY_PROBE,          fault_text = "PRB200" },
                      { fault = cGENERATION_TIMEOUT_ERROR,        fault_text = "SYS200" },
                      { fault = cPCB_AMBIENT_TOO_HIGH_ERROR,      fault_text = "PM100"  },
                      { fault = cANOLYTE_CELL_SHORT,              fault_text = "CEL150" },
                    }
           
local extended_fault_table = { { fault = cCATHOLYTE_CELL_SHORT,            fault_text = "CEL160" },
                               { fault = cTOTAL_CELL_CURRENT_TOO_HIGH,     fault_text = "CEL170" },
                               { fault = cCATHOLYTE_CELL_FET_SHORT,        fault_text = "PM210"  },
                               { fault = cANOLYTE_CELL_FET_SHORT,          fault_text = "PM200"  },
                             }

gui_state = cGUI_STATE_NOSTATE
local app_state = cAPP_STATE_NONE
local wear_code = cNO_WEAR_CODE_ACTIVE
local fault_code = cNO_ERRORS
local extended_fault_code = cNO_ERRORS
local warning = cNO_ERRORS


local function do_app_state(state)
  local mapargs = {}
  local new_state = bytes_to_int(state)
  
  if new_state == cAPP_STATE_STARTUP then --Startup
    if new_state ~= app_state then
      app_state = new_state
      mapargs.menu_screen = "blank_screen"
      goto_menu_screen(mapargs)  
    end
  elseif new_state == cAPP_STATE_IDLE then --idle
    if new_state ~= app_state then
      app_state = new_state
      go_home(mapargs)
    end
  elseif new_state == cAPP_STATE_WAIT then -- refresh unit
    if new_state ~= app_state then
      app_state = new_state
      mapargs.menu_screen = "Wait_screen"
      goto_menu_screen(mapargs)
    end
  elseif new_state == cAPP_STATE_REGEN then -- regen
    if new_state ~= app_state then
      app_state = new_state
      mapargs.menu_screen = "Regen_Screen"
      goto_menu_screen(mapargs)
    end      
  elseif new_state == cAPP_STATE_SOLUTION then -- solution
    if new_state ~= app_state then
      app_state = new_state
      mapargs.menu_screen = "Gen_Sol_Screen"
      goto_menu_screen(mapargs)
    end 
  elseif new_state == cAPP_STATE_INFO then -- status
    if new_state ~= app_state then
      app_state = new_state
      mapargs.menu_screen = "Status_screen"
      goto_menu_screen(mapargs)
    end
  elseif new_state == cAPP_STATE_QUANTITY_DISPENSED then -- Quantantity_Dispensed screen
    if new_state ~= app_state then
      app_state = new_state
      mapargs.menu_screen = "Quantity_dispensed_screen"
      goto_menu_screen(mapargs)
    end 
  elseif new_state == cAPP_STATE_TIMEOUT then -- Timeout
    if new_state ~= app_state then
      app_state = new_state
      go_home(mapargs)
    end
  elseif new_state == cAPP_STATE_ERROR then -- Error
    if new_state ~= app_state then
      app_state = new_state
      mapargs.menu_screen = "Warning_screen"
      goto_menu_screen(mapargs)
    end
  elseif new_state == cAPP_STATE_WARNING then -- Warning
    if new_state ~= app_state then
      app_state = new_state
      mapargs.menu_screen = "Warning_screen"
      goto_menu_screen(mapargs)
    end      
  end
end

local function setup_ecell_wear_reset_screen(mapargs)
  local data = {}
  
  data["wear_reset_header.window_box.text"] = ""
  data["wear_reset_header.window_box.image"] = "images/ecell_icon.png"
  
  gre.set_data(data)
end
  
local function setup_manifold_wear_reset_screen(mapargs)
  local data = {}
  
  data["wear_reset_header.window_box.text"] = ""
  data["wear_reset_header.window_box.image"] = "images/manifold_icon.png"
  
  gre.set_data(data)
end

local function setup_ph_wear_reset_screen(mapargs)
  local data = {}
  
  data["wear_reset_header.window_box.text"] = "pH"
  data["wear_reset_header.window_box.image"] = ""
  
  gre.set_data(data)
end

function initialize_user_interface(mapargs)

  initialize_working_mode(mapargs) 

  register_cb(pascal_Display_ID.PO_APP_STATE, do_app_state)
  register_cb(pascal_Display_ID.PO_MACHINE_FAULT, display_machine_fault)
  register_cb(pascal_Display_ID.PO_COMPONENT_WEAR_WARNING, display_component_wear_warning)
  register_cb(pascal_Display_ID.PO_MACHINE_WARNING, display_machine_warning)
  register_cb(pascal_Display_ID.PO_GUI_STATE, set_gui_state)
  register_cb(pascal_Display_ID.PO_WEAR_RESET, display_component_wear_reset_confirm)
  
  mapargs.enable = "propel"
  enable_machine(mapargs)
  
  gre.send_event("cycle_quantity_timer")

  
end

function set_gui_state(mapargs)
  gui_state = bytes_to_int(mapargs)
end
  
  

function enable_machine(mapargs)
  local data = {}
  local request = 0
 
  data = int_to_bytes(request, 2)
  pascal_send(pascal_Input_ID.PA_APP_REQUEST, data)
end

function do_refresh(mapargs) 
  local data = {}
  local request = cPASCAL_REFRESH_REQ
  
  data = int_to_bytes(request, 2)
  pascal_send(pascal_Input_ID.PA_RESET_BUTTON, data)
end

function display_component_wear_warning(mapargs)
  wear_code = bytes_to_int(mapargs)
  
  go_home(mapargs)
end

function get_component_wear_code(mapargs)
  return wear_code
end

function display_component_wear_reset_confirm(mapargs)
  local wear_component = bytes_to_int(mapargs)
  if wear_component == cWEAR_COMPONENT_ECELL then
    setup_ecell_wear_reset_screen()
  elseif wear_component == cWEAR_COMPONENT_MANIFOLD then
    setup_manifold_wear_reset_screen()
  else
    setup_ph_wear_reset_screen()
  end
  
  mapargs.menu_screen = "Wear_Reset_Screen"
  goto_menu_screen(mapargs)
end

function reset_wear_response(mapargs) 
  local msg_data = int_to_bytes(cPASCAL_WEAR_RESET_RESP, 2)

  if mapargs.confirm_response == "yes" then
    msg_data[#msg_data+1] = cWEAR_CONFIRM_ACCEPTED
  else
    msg_data[#msg_data+1] = cWEAR_CONFIRM_DENIED
  end
  
  pascal_send(pascal_Input_ID.PA_APP_REQUEST, msg_data)
  
  mapargs.menu_screen = "Status_screen"
  goto_menu_screen(mapargs)
end

function display_machine_fault(fault_data)
  fault_code = bytes_to_int({fault_data[1],fault_data[2],fault_data[3],fault_data[4]})
  extended_fault_code = bytes_to_int({fault_data[5],fault_data[6],fault_data[7],fault_data[8]})
  update_fault_and_warning_icon()
end   

function display_machine_warning(warning_data)
  warning = bytes_to_int(warning_data)
  update_fault_and_warning_icon()
end

function update_fault_and_warning_icon(mapargs)
  local data = {}
  
  data["warning_layer.warning_icon.grd_hidden"] = 0
  data["warning_layer.fault_code.grd_hidden"] = 0
  
--  print("number 4")
--  print(mapargs[4])
  print(string.format("0x%08x%08x",extended_fault_code,fault_code)) 
  if fault_code ~= cNO_ERRORS then
    for i=1,#fault_table do
      local bytes = bit32.band(fault_table[i].fault,fault_code)
      if bit32.band(fault_table[i].fault,fault_code) ~= 0 then
        data["warning_layer.fault_code.text"] = fault_table[i].fault_text
      end
    end  
  end
  
  if extended_fault_code ~= cNO_ERRORS then
    for i=1,#extended_fault_table do
      local bytes = bit32.band(extended_fault_table[i].fault,extended_fault_code)
      if bit32.band(extended_fault_table[i].fault,extended_fault_code) ~= 0 then
        data["warning_layer.fault_code.text"] = extended_fault_table[i].fault_text
      end
    end  
  end
 
  if fault_code == cNO_ERRORS  and extended_fault_code == cNO_ERRORS and warning == cNO_ERRORS then
    data["warning_layer.warning_icon.image"] = ""
    data["warning_layer.fault_code.grd_hidden"] = 1
    data["warning_layer.fault_code.text"] = ""
  elseif fault_code == cNO_WATER_FLOW_ERROR then
    data["warning_layer.warning_icon.image"] = "images/Water_Off.png"
  elseif fault_code == cWATER_TEMP_TOO_HIGH_ERROR then 
    data["warning_layer.warning_icon.image"] = "images/Hot_Water.png"  
  elseif fault_code == cREPLACE_PH_PROBE then
    data["warning_layer.warning_icon.image"] = "images/perodic_maintainence.png"  
  elseif (fault_code == cINVALID_SWITCHES_ERROR or fault_code == cPOWER_SUPPLY_VOLTAGE_ERROR or
         fault_code == cWATER_TEMP_TOO_LOW_ERROR or fault_code == cGENERATION_TIMEOUT_ERROR or
         fault_code == cPH_TOO_LOW or fault_code == cPH_TOO_HIGH or
         fault_code == cPCB_AMBIENT_TOO_HIGH_ERROR or fault_code == cPM_COMMUNICATIONS_ERROR or
         fault_code == cWS_COMMUNICATIONS_ERROR or fault_code == cWS_SWITCH_ERROR or
         fault_code == cWS_MOTOR_ERROR or fault_code == cWS_REGEN_ERROR or
         fault_code == cWS_BRINE_FILL_ERROR or fault_code == cWS_BRINE_SENSOR_ERROR or
         fault_code == cTSK_STARTUP_ERROR or fault_code == cLEVEL_SWITCH_UNPLUGGED or
         fault_code == cE_TAPE_VOLTAGE_TOO_HIGH or fault_code == cPH_SPI_BUS_READ_ERROR or
         fault_code == cE_TAPE_VOLTAGE_TOO_LOW or fault_code == cTOTAL_CELL_CURRENT_TOO_LOW or
         fault_code == cANOLYTE_CELL_CURRENT_TOO_LOW or fault_code == cCATHOLYTE_CELL_CURRENT_TOO_LOW or
         fault_code == cANOLYTE_CELL_CURRENT_TOO_HIGH or fault_code == cCATHOLYTE_CELL_CURRENT_TOO_HIGH or
         fault_code == cBAD_CONDUCTIVITY_PROBE or fault_code == cCATHOLYTE_CELL_SHORT or
         extended_fault_code == cCATHOLYTE_CELL_SHORT or extended_fault_code == cTOTAL_CELL_CURRENT_TOO_HIGH or
         extended_fault_code == cANOLYTE_CELL_FET_SHORT or extended_fault_code == cCATHOLYTE_CELL_FET_SHORT) then
    data["warning_layer.warning_icon.image"] = "images/Red_Wrench.png" 
  elseif warning == cHOT_WATER_DETECTED then
    data["warning_layer.warning_icon.image"] = "images/Hot_Water.png"
    data["warning_layer.fault_code.grd_hidden"] = 1
    data["warning_layer.fault_code.text"] = ""
  elseif warning == cLOW_SALT_DETECTED then 
    data["warning_layer.warning_icon.image"] = "images/LowSalt.png" 
    data["warning_layer.fault_code.grd_hidden"] = 1
    data["warning_layer.fault_code.text"] = ""    
  end
  
  if fault_code == cNO_ERRORS and extended_fault_code == cNO_ERRORS and warning ~= cNO_ERRORS then
    gre.send_event("flash_warning")
  else
    gre.send_event("flash_warning_stop")
  end
  
  gre.set_data(data)
  
end
