-- timers
local wait_screen_timer = nil
local wait_screen_index = 1
local quantity_dispensed_screen_timer = nil
local quantity_dispensed_toggle = 0

-- Quantity dispensed screen
local anolyte_liters = 0 -- liters x10
local anolyte_gallons = 0
local catholyte_liters = 0 -- liters x10
local catholyte_gallons = 0

-- status screen data
local ecell_percent = 0
local manifold_percent = 0
local ph_percent = 0

-- service screen 1 data
local anolyte_voltage_x10 = 0
local anolyte_current_x100 = 0
local ecell_supply_voltage_x10 = 0
local catholyte_voltage_x10 = 0
local catholyte_current_x100 = 0
local ambient_temp = 0
local anolyte_temp = 0
local catholyte_temp = 0

--service screen 2 data
local probe_1_conductivity = 0
local probe_2_conductivity = 0
local ph_probe_data = 0
local spm_data = 0
local conductivity_probe_delta = 0
local probe_1_average = 0
local probe_2_average = 0

-- service screen 3 data
local catholyte_tank_full_switch_status = 0
local catholyte_tank_empty_switch_status = 0
local anolyte_tank_full_switch_status = 0
local anolyte_tank_empty_switch_status = 0

--service screen 4 data
local probe_1_mkx1000 = 0
local probe_1_highx10 = 0
local probe_1_lowx10 = 0
local probe_2_mkx1000 = 0
local probe_2_highx10 = 0
local probe_2_lowx10 = 0

-- service screen 5 and 6 data
local softener_regen_state = 0
local softener_status = 0
local softener_dwell_time_sec = 0
local softener_dwell_time_min = 0
local softener_time_remaining_sec = 0
local softener_time_remaining_min = 0
local softener_brine_voltage_x100 = 0
local softener_brine_level_x100 = 0
local softener_full_threshold_x100 = 0
local softener_empty_threshold_x100 = 0

-- service screen 7
-- TODO: version screen
-- TODO: do we need to add linux versions here or should we do something else?

-- service screen 8
local water_temp = 0
local accumulated_generated = 0

-- service screen 9
local signal_strength = 0
local cell_serial_number = 0
local sim_number = 0
local phone_number_msisdn = 0
local registered_to_tower = 0

-- service screen 10
local cond_control_task = 0
local ph_control_task = 0
local gui_manager_task = 0
local read_input_task = 0
local iso_components_task = 0
local process_manager_task = 0
local pm_comlink_task = 0
local ws_comlink_task = 0
local pm_obd_task = 0
local ws_obd_task = 0

local animate_index = 1
local animate_timer = nil


function initialize_working_mode(mapargs)
  register_cb(pascal_Display_ID.PO_ANOLYTE_GENERATED, set_anolyte_generated)
  register_cb(pascal_Display_ID.PO_CATHOLYTE_GENERATED, set_catholyte_generated)
  
  register_cb(pascal_Display_ID.PO_ECELL_PERCENT, set_ecell_percent)
  register_cb(pascal_Display_ID.PO_MANIFOLD_PERCENT, set_manifold_percent)
  register_cb(pascal_Display_ID.PO_PH_PERCENT, set_ph_percent)
  register_cb(pascal_Display_ID.PO_ECELL_ACCUM, set_ecell_accum)
  register_cb(pascal_Display_ID.PO_MANIFOLD_ACCUM, set_manifold_accum)
  register_cb(pascal_Display_ID.PO_PH_ACCUM, set_ph_timer)  

  register_cb(pascal_Display_ID.PO_ANOLYTE_VOLTAGE, set_anolyte_voltage)
  register_cb(pascal_Display_ID.PO_ANOLYTE_CURRENT, set_anolyte_current)
  register_cb(pascal_Display_ID.PO_CATHOLYTE_VOLTAGE, set_catholyte_voltage)
  register_cb(pascal_Display_ID.PO_CATHOLYTE_CURRENT, set_catholyte_current)
  register_cb(pascal_Display_ID.PO_ECELL_SUPPLY_VOLTAGE, set_ecell_supply_voltage)
  register_cb(pascal_Display_ID.PO_AMBIENT_TEMP, set_ambient_temp)
  register_cb(pascal_Display_ID.PO_ANOLYTE_TEMP, set_anolyte_temp)
  register_cb(pascal_Display_ID.PO_CATHOLYTE_TEMP, set_catholyte_temp)
  
  register_cb(pascal_Display_ID.PO_PROBE1_CONDUCTIVITY, set_probe_1_conductivity)
  register_cb(pascal_Display_ID.PO_PROBE2_CONDUCTIVITY, set_probe_2_conductivity)
  register_cb(pascal_Display_ID.PO_PH_PROBE_DATA, set_ph_probe_data)
  register_cb(pascal_Display_ID.PO_SPM_DATA, set_spm_data)
  register_cb(pascal_Display_ID.PO_PROBE_DELTA, set_probe_delta)
  register_cb(pascal_Display_ID.PO_PROBE1_AVERAGE, set_probe_1_average)
  register_cb(pascal_Display_ID.PO_PROBE2_AVERAGE, set_probe_2_average)
  
  register_cb(pascal_Display_ID.PO_CATHOLYTE_FULL_SWITCH, set_catholyte_full_switch_status)
  register_cb(pascal_Display_ID.PO_CATHOLYTE_EMPTY_SWITCH, set_cathoyte_empty_switch_status)
  register_cb(pascal_Display_ID.PO_ANOLYTE_FULL_SWITCH, set_anolyte_full_switch_status)
  register_cb(pascal_Display_ID.PO_ANOLYTE_EMPTY_SWITCH, set_anolyte_empty_switch_status)
  
  register_cb(pascal_Display_ID.PO_PROBE1_MK, set_probe_1_mk_x1000)
  register_cb(pascal_Display_ID.PO_PROBE1_HIGH, set_probe_1_high_x10)
  register_cb(pascal_Display_ID.PO_PROBE1_LOW, set_probe_1_low_x10)
  register_cb(pascal_Display_ID.PO_PROBE2_MK, set_probe_2_mk_x1000)
  register_cb(pascal_Display_ID.PO_PROBE2_HIGH, set_probe_2_high_x10)
  register_cb(pascal_Display_ID.PO_PROBE2_LOW, set_probe_2_low_x10)  
  
  register_cb(pascal_Display_ID.PO_REGEN_STATE, set_regen_state)
  register_cb(pascal_Display_ID.PO_SOFTENER_STATUS, set_softener_status)
  register_cb(pascal_Display_ID.PO_DWELL_TIME, set_dwell_time)
  register_cb(pascal_Display_ID.PO_SOFTENER_TIME_REMAINING, set_time_remaining)
  register_cb(pascal_Display_ID.PO_BRINE_VOLTAGE, set_brine_voltage_x100)
  register_cb(pascal_Display_ID.PO_BRINE_LEVEL, set_brine_level_x100)
  register_cb(pascal_Display_ID.PO_BRINE_FULL_THRESHOLD, set_brine_full_threshold_x100)
  register_cb(pascal_Display_ID.PO_BRINE_EMPTY_THRESHOLD, set_brine_empty_threshold_x100)
  
  register_cb(pascal_Display_ID.PO_WATER_TEMP, set_water_temp)
  
  register_cb(pascal_Display_ID.PO_SIGNAL_STRENGTH, set_signal_strength)
  register_cb(pascal_Display_ID.PO_CELL_SERIAL, set_cell_serial_number)
  register_cb(pascal_Display_ID.PO_SIM_NUMBER, set_sim_number)
  register_cb(pascal_Display_ID.PO_PHONE_MSIDSN_NUMBER, set_phone_msidsn_number)
  register_cb(pascal_Display_ID.PO_CELL_REGISTERED, set_cell_registered)
  
  register_cb(pascal_Display_ID.PO_TASK_STATUS, set_task_status)
  
  register_cb(pascal_Display_ID.PO_REV_INFO, set_revisions)
  register_cb(pascal_Display_ID.PO_SECOND_BOOT, set_second_bootloader_state)
  
end

function set_anolyte_voltage(mapargs)
  local data = {}
  
  anolyte_voltage_x10 = bytes_to_int(mapargs)
  local str = string.format("%2.1fV", anolyte_voltage_x10 / 10)
  
  data["service_page_1.anolyte_voltage_current.voltage_text"] = str
  gre.set_data(data)  
end

function set_anolyte_current(mapargs)
  local data = {}
  
  anolyte_current_x100 = bytes_to_int(mapargs)
  local str = string.format("(%2.1fA)", anolyte_current_x100 / 100)
    
  data["service_page_1.anolyte_voltage_current.current_text"] = str
  gre.set_data(data)
end

function set_catholyte_voltage(mapargs)
  local data = {}
  
  catholyte_voltage_x10 = bytes_to_int(mapargs)
  local str = string.format("%2.1fV", catholyte_voltage_x10 / 10)
    
  data["service_page_1.catholyte_voltage_current.voltage_text"] = str
  gre.set_data(data)
end

function set_catholyte_current(mapargs)
  local data = {}
  
  catholyte_current_x100 = bytes_to_int(mapargs)

  local str = string.format("(%2.1fA)", catholyte_current_x100 / 100)
  
  data["service_page_1.catholyte_voltage_current.current_text"] = str
  gre.set_data(data)
end

function set_ecell_supply_voltage(mapargs)
  local data = {}
  
  ecell_supply_voltage_x10 = bytes_to_int(mapargs)
  local str = string.format("%02.1fV", ecell_supply_voltage_x10 / 10)
  
  data["service_page_1.ecell_voltage.text"] = str
  gre.set_data(data)
end

function set_ambient_temp(mapargs)
  local data = {}
  
  ambient_temp = bytes_to_int(mapargs)
  local str = string.format("%02dC", ambient_temp / 10)
  
  data["service_page_1.ambient_temp.text"] = str
  gre.set_data(data)
end

function set_anolyte_temp(mapargs)
  local data = {}
  
  anolyte_temp = bytes_to_int(mapargs)
  local str = string.format("%02dC / %02dC", anolyte_temp / 10, catholyte_temp / 10)
  
  data["service_page_1.anol_cath_temp.text"] = str
  gre.set_data(data)
end

function set_catholyte_temp(mapargs)
  local data = {}
  
  catholyte_temp = bytes_to_int(mapargs)
  local str = string.format("%02dC / %02dC", anolyte_temp / 10, catholyte_temp / 10)
  
  data["service_page_1.anol_cath_temp.text"] = str
  gre.set_data(data)
end

function set_anolyte_generated(mapargs)
  local data = {}
  
  anolyte_liters = (bytes_to_int(mapargs))/100
  anolyte_gallons = anolyte_liters * 0.264172

  gre.set_data(data)
end

function set_catholyte_generated(mapargs)
  local data = {}
  
  catholyte_liters = (bytes_to_int(mapargs))/100
  catholyte_gallons = catholyte_liters * 0.264172
  
  data["service_page_8.msc_gen.text"] = string.format("%7.1f gals",catholyte_gallons)
  gre.set_data(data)
end

function set_probe_1_conductivity(mapargs)
  local data = {}
  
  probe_1_conductivity = bytes_to_int(mapargs)
  local str = string.format("%04.0fuS", probe_1_conductivity / 10)
  
  data["service_page_2.conductivity_probe_1.text"] = str
  gre.set_data(data)
end

function set_probe_2_conductivity(mapargs)
  local data = {}
  
  probe_2_conductivity = bytes_to_int(mapargs)
  local str = string.format("%04.0fuS", probe_2_conductivity / 10)
  
  data["service_page_2.conductivity_probe_2.text"] = str
  gre.set_data(data)
end

function set_ph_probe_data(mapargs)
  local data = {}
  
  ph_probe_data = bytes_to_int(mapargs)
  local str = string.format("%02.1f", ph_probe_data / 10)
  
  data["service_page_2.ph_probe.text"] = str
  gre.set_data(data)
end

function set_spm_data(mapargs)
  local data = {}
  
  spm_data = bytes_to_int(mapargs)
  local str = string.format("%d", spm_data)
  
  data["service_page_2.spm.text"] = str
  gre.set_data(data)
end

function set_probe_delta(mapargs)
  local data = {}
  local value = {}
  local str = ""
  
  value[1] = mapargs[1]
  value[2] = mapargs[2]
  
  conductivity_probe_delta = bytes_to_int(value)
  
  if mapargs[3] == cPASCAL_SIGN_NEGATIVE then
    str = string.format("-%4.0f", conductivity_probe_delta / 10)
  else
    str = string.format("%4.0f", conductivity_probe_delta / 10)
  end
  
  data["service_page_2.delta.text"] = str
  gre.set_data(data)  
end

function set_probe_1_average(mapargs)
  local data = {}
  
  probe_1_average = bytes_to_int(mapargs)
  local str = string.format("%04.0f", probe_1_average / 10)
  
  data["service_page_2.probe1_average.text"] = str
  gre.set_data(data)
end

function set_probe_2_average(mapargs)
  local data = {}
  
  probe_2_average = bytes_to_int(mapargs)
  local str = string.format("%04.0f", probe_2_average / 10)
  
  data["service_page_2.probe2_average.text"] = str
  gre.set_data(data)
end

function set_catholyte_full_switch_status(mapargs)
  local data = {}
  catholyte_tank_full_switch_status = bytes_to_int(mapargs)
  if catholyte_tank_full_switch_status == 1 then
    data["service_page_3.catholyte_full_switch.text"] = "Cath FULL Sw. Full"
  else
    data["service_page_3.catholyte_full_switch.text"] = "Cath FULL Sw. Not Full"
  end
  
  gre.set_data(data)
end

function set_cathoyte_empty_switch_status(mapargs)
  local data = {}
  catholyte_tank_empty_switch_status = bytes_to_int(mapargs)
  if catholyte_tank_empty_switch_status == 1 then
    data["service_page_3.catholyte_empty_switch.text"] = "Cath EMPTY Sw. Empty"
  else
    data["service_page_3.catholyte_empty_switch.text"] = "Cath EMPTY Sw. Not Empty"
  end
  
  gre.set_data(data)
end

function set_anolyte_full_switch_status(mapargs)
  local data = {}
  anolyte_tank_full_switch_status = bytes_to_int(mapargs)
  if anolyte_tank_full_switch_status == 1 then
    data["service_page_3.anolyte_full_switch.text"] = "Anol FULL Sw. Full"
  else
    data["service_page_3.anolyte_full_switch.text"] = "Anol FULL Sw. Not Full"
  end
  
  gre.set_data(data)
end

function set_anolyte_empty_switch_status(mapargs)
  local data = {}
  anolyte_tank_empty_switch_status = bytes_to_int(mapargs)
  if anolyte_tank_empty_switch_status == 1 then
    data["service_page_3.anolyte_empty_switch.text"] = "Anolyte EMPTY Sw. Empty"
  else
    data["service_page_3.anolyte_empty_switch.text"] = "Anolyte EMPTY Sw. Not Empty"
  end
  
  gre.set_data(data)
end

function set_probe_1_mk_x1000(mapargs)
  local data = {}
  
  probe_1_mkx1000 = bytes_to_int(mapargs)
  local str = string.format("%04.3f", probe_1_mkx1000 / 1000)
  
  data["service_page_4.probe1_Mk.text"] = str
  gre.set_data(data)
end

function set_probe_1_high_x10(mapargs)
  local data = {}
  
  probe_1_highx10 = bytes_to_int(mapargs)
  local str = string.format("%04.1f", probe_1_highx10 / 10)
  
  data["service_page_4.probe1_Hi.text"] = str
  gre.set_data(data) 
end

function set_probe_1_low_x10(mapargs)
  local data = {}
  
  probe_1_lowx10 = bytes_to_int(mapargs)
  local str = string.format("%04.1f", probe_1_lowx10 / 10)
  
  data["service_page_4.probe1_Low.text"] = str
  gre.set_data(data) 
end

function set_probe_2_mk_x1000(mapargs)
  local data = {}
  
  probe_2_mkx1000 = bytes_to_int(mapargs)  
  local str = string.format("%04.3f", probe_2_mkx1000 / 1000)
  
  data["service_page_4.probe2_Mk.text"] = str
  gre.set_data(data)
end

function set_probe_2_high_x10(mapargs)
  local data = {}
  
  probe_2_highx10 = bytes_to_int(mapargs)
  local str = string.format("%04.1f", probe_2_highx10 / 10)
  
  data["service_page_4.probe2_Hi.text"] = str
  gre.set_data(data) 
end

function set_probe_2_low_x10(mapargs)
  local data = {}
  
  probe_2_lowx10 = bytes_to_int(mapargs)
  local str = string.format("%04.1f", probe_2_lowx10 / 10)
  
  data["service_page_4.probe2_Low.text"] = str
  gre.set_data(data) 
end

function set_regen_state(mapargs)
  local data = {}
  
  softener_regen_state = bytes_to_int(mapargs)
  if softener_regen_state == 2 then
    data["service_page_5.ws_state.text"] = "FILL"
  elseif softener_regen_state == 3 then
    data["service_page_5.ws_state.text"] = "BRINE DRAW"
  elseif softener_regen_state == 4 then
    data["service_page_5.ws_state.text"] = "BACKWASH"
  elseif softener_regen_state == 5 then
    data["service_page_5.ws_state.text"] = "FAST RNSE"
  elseif softener_regen_state == 7 then
    data["service_page_5.ws_state.text"] = "BRINE FILL"
  else
    data["service_page_5.ws_state.text"] = "SERVICE"
  end
  
  gre.set_data(data)
end

function set_softener_status(mapargs)
  local data = {}
  
  softener_status = bytes_to_int(mapargs)
  
  data["service_page_5.ws_status.text"] = string.format("0x%08X",softener_status)
  
  gre.set_data(data)
end

function set_dwell_time(mapargs)
  local data = {}
  
  softener_dwell_time_sec = bytes_to_int(mapargs)
  softener_dwell_time_min = softener_dwell_time_sec / 60
  
  data["service_page_5.ws_dwell_time.text"] = string.format("%ds (%d min)",softener_dwell_time_sec, softener_dwell_time_min)
  
  gre.set_data(data)
end

function set_time_remaining(mapargs)
  local data = {}
  
  softener_time_remaining_sec = bytes_to_int(mapargs)
  data["service_page_5.ws_remain_time.text"] = string.format("%ds (%d min)",softener_time_remaining_sec, softener_time_remaining_sec / 60)
  
  gre.set_data(data)
end

function set_brine_voltage_x100(mapargs)
  local data = {}
  
  softener_brine_voltage_x100 = bytes_to_int(mapargs)
  data["service_page_5.ws_brine_voltage.text"] = string.format("%2.2fV",softener_brine_voltage_x100 / 100)
  data["service_page_6.ws_brine_voltage.text"] = string.format("%2.2fV",softener_brine_voltage_x100 / 100)
  
  gre.set_data(data)
end

function set_brine_level_x100(mapargs)
  local data = {}
  
  softener_brine_level_x100 = bytes_to_int(mapargs)
  data["service_page_6.ws_full_level.text"] = string.format("%2.2fV",softener_brine_level_x100 / 100)
  
  gre.set_data(data)
end

function set_brine_full_threshold_x100(mapargs)
  local data = {}
  
  softener_full_threshold_x100 = bytes_to_int(mapargs)
  data["service_page_6.ws_full_threshold.text"] = string.format("%2.2fV",softener_full_threshold_x100 / 100)
  
  gre.set_data(data)
end

function set_brine_empty_threshold_x100(mapargs)
  local data = {}
  
  softener_empty_threshold_x100 = bytes_to_int(mapargs)
  data["service_page_6.ws_empty_threshold.text"] = string.format("%2.2fV",softener_empty_threshold_x100 / 100)
  
  gre.set_data(data)
end

function set_water_temp(mapargs)
  local data = {}
  
  water_temp = bytes_to_int(mapargs)
  data["service_page_8.water_temp.text"] = water_temp
  
  gre.set_data(data)
end

function set_task_status(mapargs)
  local data = {}
  
  data["service_page_10.cond_control_task.text"] = mapargs[1]
  data["service_page_10.ph_control_task.text"] = mapargs[2]
  data["service_page_10.gui_manager_task.text"] = mapargs[3]
  data["service_page_10.read_inputs_task.text"] = mapargs[4]
  data["service_page_10.iso_components_task.text"] = mapargs[5]
  data["service_page_10.process_manager_task.text"] = mapargs[6]
  data["service_page_10.pm_comlink_task.text"] = mapargs[7]
  data["service_page_10.ws_comlink_task.text"] = mapargs[8]
  data["service_page_10.pm_obd_task.text"] = mapargs[9]
  data["service_page_10.ws_obd_task.text"] = mapargs[10]
  
  gre.set_data(data)
end

function set_ecell_percent(mapargs)
  local data = {}
  ecell_percent = bytes_to_int(mapargs)
  
  data["service_page_8.cell_wear.percent_text"] = string.format("%d%%",ecell_percent)
  
  if (ecell_percent > 75) then
    data["status_layer.ecell_percent_status.image"] = "images/Green_bar_4.png"
  elseif (ecell_percent > 50) then
    data["status_layer.ecell_percent_status.image"] = "images/Green_bar_3.png"
  elseif (ecell_percent > 25) then
    data["status_layer.ecell_percent_status.image"] = "images/Green_bar_2.png"
  elseif (ecell_percent > 10) then
    data["status_layer.ecell_percent_status.image"] = "images/Yellow_bar_depleted.png"
  else 
    data["status_layer.ecell_percent_status.image"] = "images/Red_bar_depleted.png"
  end
  
  gre.set_data(data)
end

function set_manifold_percent(mapargs)
  local data = {}
  manifold_percent = bytes_to_int(mapargs)
  
  data["service_page_8.manifold_wear.percent_text"] = string.format("%d%%",manifold_percent)
  
  if (manifold_percent > 75) then
    data["status_layer.manifold_percent_status.image"] = "images/Green_bar_4.png"
  elseif (manifold_percent > 50) then
    data["status_layer.manifold_percent_status.image"] = "images/Green_bar_3.png"
  elseif (manifold_percent > 25) then
    data["status_layer.manifold_percent_status.image"] = "images/Green_bar_2.png"
  elseif (manifold_percent > 10) then
    data["status_layer.manifold_percent_status.image"] = "images/Yellow_bar_depleted.png"
  else 
    data["status_layer.manifold_percent_status.image"] = "images/Red_bar_depleted.png"
  end
  
  gre.set_data(data)
  
end

function set_ph_percent(mapargs)
  local data = {}
  ph_percent = bytes_to_int(mapargs)
  
  data["service_page_8.probe_wear.percent_text"] = string.format("%d%%",ph_percent)
  
  if (ph_percent > 75) then
    data["status_layer.ph_percent_status.image"] = "images/Green_bar_4.png"
  elseif (ph_percent > 50) then
    data["status_layer.ph_percent_status.image"] = "images/Green_bar_3.png"
  elseif (ph_percent > 25) then
    data["status_layer.ph_percent_status.image"] = "images/Green_bar_2.png"
  elseif (ph_percent > 10) then
    data["status_layer.ph_percent_status.image"] = "images/Yellow_bar_depleted.png"
  else 
    data["status_layer.ph_percent_status.image"] = "images/Red_bar_depleted.png"
  end
  
  gre.set_data(data)
  
end

function set_ph_timer(mapargs)
  local data = {}
  local ph_timer = bytes_to_int(mapargs)
  
  data["service_page_8.probe_wear.gal_text"] = string.format("(%d mins.)",ph_timer)

  gre.set_data(data)
end

function set_ecell_accum(mapargs)
  local data = {}
  local ecell_accum = bytes_to_int(mapargs)
  
  data["service_page_8.cell_wear.gal_text"] = string.format("(%dgals.)",(ecell_accum * 0.264172) / 100)

  gre.set_data(data)
end

function set_manifold_accum(mapargs)
  local data = {}
  local manifold_accum = bytes_to_int(mapargs)
  
  data["service_page_8.manifold_wear.gal_text"] = string.format("(%dgals.)",(manifold_accum * 0.264172) / 100)

  gre.set_data(data)
end

function set_revisions(revisions)
  do_rev_info(revisions)
end

function set_second_bootloader_state(mapargs)
  local data = {}
  local bootloader_present = bytes_to_int(mapargs)
  
  if bootloader_present == 1 then
    data["service_page_7.bootloader_present_text"] = "PRESENT"
  else
    data["service_page_7.bootloader_present_text"] = "NOT PRESENT"
  end
  gre.set_data(data)

end

function set_cell_registered(mapargs)
  local data = {}
  local cell_registered = bytes_to_int(mapargs)
  
  if cell_registered == 1 then
    data["service_page_9.tower.text"] = "Registered to Tower"
  else
    data["service_page_9.tower.text"] = "NOT Registered to Tower"
  end
  gre.set_data(data)

end


function set_signal_strength(mapargs)
  local data = {}
  local signal_strength = bytes_to_int(mapargs)

  data["service_page_9.cell_strength.text"] = string.format("%d",signal_strength)

  gre.set_data(data)

end 

function set_cell_serial_number(mapargs)
  local data = {}
                                                    
  data["service_page_9.cell_serial.text"] = string.format("%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c",
                                           mapargs[1], mapargs[2], mapargs[3], mapargs[4], mapargs[5], mapargs[6], mapargs[7], 
                                           mapargs[8], mapargs[9], mapargs[10], mapargs[11], mapargs[12], mapargs[13], mapargs[14], 
               
                                           mapargs[15], mapargs[16], mapargs[17], mapargs[18], mapargs[19], mapargs[20], mapargs[21], 
                                           mapargs[22], mapargs[23], mapargs[24], mapargs[25], mapargs[26], mapargs[27], mapargs[28], 
                                           mapargs[29], mapargs[30], mapargs[31], mapargs[32], mapargs[33], mapargs[34], mapargs[35], 
                                           mapargs[36], mapargs[37], mapargs[38], mapargs[39], mapargs[40] )
  
  gre.set_data(data)

end
 
function set_sim_number(mapargs)
  local data = {}
                                                         
  data["service_page_9.sim_header.text"] = string.format("%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c",
                                           mapargs[1], mapargs[2], mapargs[3], mapargs[4], mapargs[5], mapargs[6], mapargs[7], 
                                           mapargs[8], mapargs[9], mapargs[10], mapargs[11], mapargs[12], mapargs[13], mapargs[14], 
                                           mapargs[15], mapargs[16], mapargs[17], mapargs[18], mapargs[19], mapargs[20], mapargs[21] )
  
  gre.set_data(data)

end


function set_phone_msidsn_number(mapargs)
  local data = {}
                                                         
  data["service_page_9.sim_phone.text"] = string.format("%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c",
                                           mapargs[1], mapargs[2], mapargs[3], mapargs[4], mapargs[5], mapargs[6], mapargs[7], 
                                           mapargs[8], mapargs[9], mapargs[10], mapargs[11], mapargs[12], mapargs[13], mapargs[14], 
                                           mapargs[15], mapargs[16], mapargs[17], mapargs[18], mapargs[19], mapargs[20] )
  
  gre.set_data(data)

end



function cycle_wait_screen(mapargs)
  local data = {}

  data["wait_bar_layer.wait_bar.image"] = string.format("images/Green_bar_%d.png",wait_screen_index)

  if (wait_screen_index >= 4) then
    wait_screen_index = 1
  else
    wait_screen_index = wait_screen_index + 1
  end
  
  gre.set_data(data)

end

function start_wait_screen(mapargs)
  gre.send_event("startWaitTimer")
  wait_screen_index = 2
end

function stop_wait_screen(mapargs)
  gre.send_event("stopWaitTimer")
end

function animate_tanks(mapargs)
  local data = {}
 
  animate_index = animate_index + 1
  if animate_index == 4 then
    animate_index = 1
  end
  
  if catholyte_tank_full_switch_status == 1 then
    data["sol_gen_layer.catholyte_hand.grd_hidden"] = 1
    data["sol_gen_layer.catholyte_tank.image"] = "images/catholyte_solution_level_3.png"
  elseif catholyte_tank_full_switch_status == 0 and catholyte_tank_empty_switch_status == 1 then
    data["sol_gen_layer.catholyte_hand.grd_hidden"] = 0
    data["sol_gen_layer.catholyte_tank.image"] = "images/catholyte_solution_level_1.png"
  else
    data["sol_gen_layer.catholyte_hand.grd_hidden"] = 1
    data["sol_gen_layer.catholyte_tank.image"] = string.format("images/catholyte_solution_level_%d.png", animate_index)
  end 
  
  if anolyte_tank_full_switch_status == 1 then
    data["sol_gen_layer.anolyte_hand.grd_hidden"] = 1
    data["sol_gen_layer.anolyte_tank.image"] = "images/anolyte_solution_level_3.png"
  elseif anolyte_tank_full_switch_status == 0 and anolyte_tank_empty_switch_status == 1 then
    data["sol_gen_layer.anolyte_hand.grd_hidden"] = 0
    data["sol_gen_layer.anolyte_tank.image"] = "images/anolyte_solution_level_1.png"
  else
    data["sol_gen_layer.anolyte_hand.grd_hidden"] = 1
    data["sol_gen_layer.anolyte_tank.image"] = string.format("images/anolyte_solution_level_%d.png", animate_index)
  end 
  
  gre.set_data(data)
end


function cycle_quantity_dispensed_screen(mapargs)
  local data = {}

  if (quantity_dispensed_toggle == 0) then -- display in liters
    quantity_dispensed_toggle = 1
    
    data["quantity_dispensed.anolyte_units.text"] = "L"
    data["quantity_dispensed.catholyte_units.text"] = "L"
    data["quantity_dispensed.anolyte_generated.text"] = string.format("%07.0f", anolyte_liters)
    data["quantity_dispensed.catholyte_generated.text"] = string.format("%07.0f",catholyte_liters)
    
  else -- display in galons
    quantity_dispensed_toggle = 0
    
    data["quantity_dispensed.anolyte_units.text"] = "GAL"
    data["quantity_dispensed.catholyte_units.text"] = "GAL"
    data["quantity_dispensed.anolyte_generated.text"] = string.format("%07.0f", anolyte_gallons)
    data["quantity_dispensed.catholyte_generated.text"] = string.format("%07.0f",catholyte_gallons)
  end
  
  gre.set_data(data)

end

